var utils = require('./utils');
var url = require('url');

module.exports.parseText = function parseText(message, kwargs) {
    kwargs = kwargs || {};
    kwargs['message'] = message;
    return kwargs;
};

module.exports.parseError = function parseError(err, kwargs, cb) {
    err.stack; // Error.prepareStackTrace is only called when stack is accessed, so access it
    utils.parseStack(err.structuredStackTrace, function(frames) {
        kwargs['message'] = err.name + ': ' + (err.message || '<no message>');
        kwargs['sentry.interfaces.Exception'] = {
            type: err.name,
            value:err.message
        };
        kwargs['sentry.interfaces.Stacktrace'] = {frames: frames};

        for (var n = 0, l = frames.length; n < l; n++) {
            if (frames[n].in_app) {
                kwargs['culprit'] = utils.getCulprit(frames[n]);
                break;
            }
        }

        cb(kwargs);
    });
};

module.exports.parseQuery = function parseQuery(query, engine, kwargs) {
    kwargs = kwargs || {};
    kwargs['message'] = query;
    kwargs['sentry.interfaces.Query'] = {
        query: query,
        engine: engine
    };
    return kwargs;
};

module.exports.parseRequest = function parseRequest(req, kwargs) {
    kwargs = kwargs || {};
    var http = {
        method: req.method,
        query_string: url.parse(req.url).query,
        headers: req.headers,
        cookies: req.cookies || '<unavailable: use cookieParser middleware>',
        data: req.body || '<unavailable: use bodyParser middleware>',
        url: (function build_absolute_url() {
            var protocol = req.socket.encrypted ? 'https' : 'http',
                host = req.headers.host || '<no host>';
            return protocol+'://'+host+req.url;
        }()),
        env: process.env
    };

    var ip = (req.headers['x-forwarded-for'] || '').split(',')[0] ||
             req.connection.remoteAddress;
    http.env.REMOTE_ADDR = ip;
    kwargs['sentry.interfaces.Http'] = http;
    return kwargs;
};
